@@ -1,7 +1,5 @@
 #######################	Conditional means - estimates of PSI_i ########################
 
-
-
 #' Estimate conditional mean and variance of individual parameters using the
 #' MCMC algorithm
 #' 
@@ -24,7 +22,13 @@
 #' nsamp>1, several chains of the MCMC algorithm are run in parallel to obtain
 #' samples from the conditional distributions, and the convergence criterion
 #' must be achieved for all chains. When nsamp>1, the estimate of the
-#' conditional mean is obtained by averaging over the different samples.
+#' conditional mean is obtained by averaging over the different samples, and the 
+#' samples from the conditional distribution are output as an array of dimension
+#' N x nb of parameters x nsamp in arrays phi.samp for the sampled phi_i and psi.samp
+#' for the corresponding psi_i. The variance of the conditional phi_i for each
+#' sample is given in a corresponding array phi.samp.var (the variance of the 
+#' conditional psi_i is not given but may be computed via the delta-method or by 
+#' transforming the confidence interval for phi_i).
 #' 
 #' The shrinkage for any given parameter for the conditional estimate is
 #' obtained as
@@ -34,20 +38,12 @@
 #' where var(eta_i) is the empirical variance of the estimates of the
 #' individual random effects, and omega(eta) is the estimated variance.
 #' 
-#' The function adds or modifies the following elements in the results:
-#' \describe{ \item{cond.mean.phi}{Conditional mean of the individual
-#' distribution of the parameters (obtained as the mean of the samples)}
-#' \item{cond.var.phi}{Conditional variance of the individual distribution of
-#' the parameters (obtained as the mean of the estimated variance of the
-#' samples)} \item{cond.shrinkage}{Estimate of the shrinkage for the
-#' conditional estimates} \item{cond.mean.eta}{Conditional mean of the
-#' individual distribution of the parameters (obtained as the mean of the
-#' samples)} \item{phi.samp}{An array with 3 dimensions, giving nsamp samples
-#' from the conditional distributions of the individual parameters}
-#' \item{phi.samp.var}{The estimated individual variances for the sampled
-#' parameters phi.samp} } A warning is output if the maximum number of
-#' iterations is reached without convergence (the maximum number of iterations
-#' is saemix.options$nbiter.saemix[2]).
+#' When the option displayProgress is set to TRUE, convergence graphs are produced
+#' They can be used assess whether the mean of the individual estimates (on the scale of the parameters) 
+#' and the mean of the SD of the random effects have stabilised over the ipar.lmcmc (defaults to 50) iterations. 
+#' The evolution of these variables for each parameter is shown as a continuous line
+#' while the shaded area represents the acceptable variation.
+#' 
 #' 
 #' @param saemixObject an object returned by the \code{\link{saemix}} function
 #' @param nsamp Number of samples to be drawn in the conditional distribution
@@ -56,7 +52,25 @@
 #' conditional estimates. Defaults to twice the total number of iterations
 #' (sum(saemixObject["options"]$nbiter.saemix)*2)
 #' @param \dots optional arguments passed to the plots. Plots will appear if
-#' the option displayProgress in the saemixObject object is TRUE
+#' the option displayProgress in the saemixObject object is TRUE (Note: currently disabled to avoid dependency on ggplot)
+#' 
+#' @return The function adds or modifies the following elements in the results slot of saemixObject:
+#' \describe{ 
+#' \item{cond.mean.phi}{Conditional mean of the individual
+#' distribution of the parameters (obtained as the mean of the samples)}
+#' \item{cond.var.phi}{Conditional variance of the individual distribution of
+#' the parameters (obtained as the mean of the estimated variance of the samples)} 
+#' \item{cond.shrinkage}{Estimate of the shrinkage for the
+#' conditional estimates} 
+#' \item{cond.mean.eta}{Conditional mean of the
+#' individual distribution of the parameters (obtained as the mean of the samples)} 
+#' \item{phi.samp}{An array with 3 dimensions, giving nsamp samples
+#' from the conditional distributions of the individual parameters}
+#' \item{phi.samp.var}{The estimated individual variances for the sampled
+#' parameters phi.samp} } A warning is output if the maximum number of
+#' iterations is reached without convergence (the maximum number of iterations
+#' is saemix.options$nbiter.saemix[2]).
+#' 
 #' @author Emmanuelle Comets <emmanuelle.comets@@inserm.fr>, Audrey Lavenu,
 #' Marc Lavielle.
 #' @seealso \code{\link{SaemixData}},\code{\link{SaemixModel}},
@@ -113,11 +127,10 @@
 #' 
 #' 
 #' @export conddist.saemix
-
 conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
   # Estimate conditional means and estimates for the individual parameters PSI_i using the MCMC algorithm
   # nsamp= number of MCMC samples
-  # kmax= max nb of iterations
+  # max.iter= max nb of iterations
   # returns an array 
   saemix.data<-saemixObject["data"]
   saemix.model<-saemixObject["model"]
@@ -150,67 +163,64 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
   omega.eta<-omega.eta-mydiag(mydiag(saemixObject["results"]["omega"][ind.eta, ind.eta]))+mydiag(domega)
   chol.omega<-chol(omega.eta)
   pres<-saemixObject["results"]["respar"]
+  args<-list(ind.ioM=ind.ioM)
   
-  # Preparing plots
-  if(saemixObject["options"]$displayProgress) {
-    plot.opt<-saemixObject["prefs"]
-    plot.opt$xlab<-"Iteration"
-    plot.opt<-replace.plot.options(plot.opt,...)
-    change.ylab<-FALSE
-    if(plot.opt$ylab!=saemixObject["prefs"]$ylab & length(plot.opt$which.par)==1) change.ylab<-TRUE
-    change.main<-FALSE
-    if(plot.opt$main!=saemixObject["prefs"]$main & length(plot.opt$which.par)==1) change.main<-TRUE
-    np<-nb.etas
-    if(length(plot.opt$mfrow)==0) {
-      n1<-round(sqrt(np))
-      n2<-ceiling(np/n1)
-      if(n1>5 | n2>5) {
-        n1<-3
-        n2<-4
-        #      cat("Changing the plot layout\n")
-      }
-      plot.opt$mfrow<-c(n1,n2)
-    }
-  }
+  # # Preparing plots
+  # if(saemixObject["options"]$displayProgress) {
+  #   plot.opt<-saemixObject["prefs"]
+  #   plot.opt$xlab<-"Iteration"
+  #   plot.opt<-replace.plot.options(plot.opt,...)
+  #   change.ylab<-FALSE
+  #   if(plot.opt$ylab!=saemixObject["prefs"]$ylab & length(plot.opt$which.par)==1) change.ylab<-TRUE
+  #   change.main<-FALSE
+  #   if(plot.opt$main!=saemixObject["prefs"]$main & length(plot.opt$which.par)==1) change.main<-TRUE
+  #   np<-nb.etas
+  #   if(length(plot.opt$mfrow)==0) {
+  #     n1<-round(sqrt(np))
+  #     n2<-ceiling(np/n1)
+  #     if(n1>5 | n2>5) {
+  #       n1<-3
+  #       n2<-4
+  #       #      cat("Changing the plot layout\n")
+  #     }
+  #     plot.opt$mfrow<-c(n1,n2)
+  #   }
+  # }
   # Simulation MCMC
   # initialisation a phiM=estimation des parametres individuels
   mean.phiM<-do.call(rbind,rep(list(saemixObject["results"]["mean.phi"]),nsamp))
   phiM<-do.call(rbind,rep(list(saemixObject["results"]["cond.mean.phi"]),nsamp))
   etaM<-phiM[,ind.eta]-mean.phiM[,ind.eta]  
-  psiM<-transphi(phiM,saemixObject["model"]["transform.par"])
-  fpred<-saemixObject["model"]["model"](psiM, IdM, XM)
-  ind.exp<-which(saemix.model["error.model"]=="exponential")
-  for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
-  gpred<-error(fpred,pres,XM$ytype)
-  DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)^2+log(gpred)
-  U.y<-colSums(DYF)
+  Dargs<-list(transform.par=saemixObject["model"]["transform.par"], structural.model=saemixObject["model"]["model"],IdM=IdM,XM=XM,yM=yM,etype.exp=which(saemixObject["model"]["error.model"] == "exponential"), modeltype=saemixObject["model"]["modeltype"])
+  Uargs<-list(ind.ioM=ind.ioM)
+  somega<-solve(omega.eta)
+  
+  #  chol.omega<-chol.omega*3
+  #  domega2<-domega2*3
+  #  somega<-somega*3
+  
+  U.y<-compute.LLy(phiM,Uargs,Dargs,DYF,pres)
   phiMc<-phiM
   
   econd<-sdcond<-array(0,dim=c(nb.parameters,N*kmax,nsamp)) # parametres individuels
   ebar<-sdbar<-array(0,dim=c(nb.parameters,kmax, nsamp)) # moyenne des parametres individuels
-  cat("Estimating the conditional mean and variance of the distribution of individual parameters\n")
-  k<-1
-  while(k<=kmax) { # Set a maximum nb of iterations
-    
+  if(saemixObject["options"]$warnings) cat("Estimating the conditional mean and variance of the distribution of individual parameters\n")
+  k<-0
+  while(k<kmax) { # Set a maximum nb of iterations
+    k<-k+1
     for(u in 1:saemixObject["options"]$nbiter.mcmc[1]) { # 1er noyau
       etaMc<-matrix(rnorm(NM*nb.etas),ncol=nb.etas)%*%chol.omega
       phiMc[,ind.eta]<-mean.phiM[,ind.eta]+etaMc
-      psiMc<-transphi(phiMc,saemixObject["model"]["transform.par"])
-      fpred<-saemixObject["model"]["model"](psiMc, IdM, XM)
-      for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
-      gpred<-error(fpred,pres,XM$ytype)
-      DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)^2+log(gpred)
-      Uc.y<-colSums(DYF)
+      Uc.y<-compute.LLy(phiMc,Uargs,Dargs,DYF,pres)
       deltau<-Uc.y-U.y
       ind<-which(deltau<(-1)*log(runif(NM)))
       etaM[ind,]<-etaMc[ind,]
       U.y[ind]<-Uc.y[ind]
     }
     
-    U.eta<-0.5*rowSums(etaM*(etaM%*%solve(omega.eta)))
+    U.eta<-0.5*rowSums(etaM*(etaM%*%somega))
     
     # Second stage
-    
     if(saemixObject["options"]$nbiter.mcmc[2]>0) {
       nt2<-nbc2<-matrix(data=0,nrow=nb.etas,ncol=1)
       nrs2<-1
@@ -219,13 +229,8 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
           etaMc<-etaM
           etaMc[,vk2]<-etaM[,vk2]+matrix(rnorm(NM*nrs2), ncol=nrs2)%*%mydiag(domega2[vk2,nrs2],nrow=1) # 2e noyau ? ou 1er noyau+permutation?
           phiMc[,ind.eta]<-mean.phiM[,ind.eta]+etaMc
-          psiMc<-transphi(phiMc,saemixObject["model"]["transform.par"])
-          fpred<-saemixObject["model"]["model"](psiMc, IdM, XM)
-          for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
-          gpred<-error(fpred,pres,XM$ytype)
-          DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)**2+log(gpred)
-          Uc.y<-colSums(DYF) # Warning: Uc.y, Uc.eta = vecteurs
-          Uc.eta<-0.5*rowSums(etaMc*(etaMc%*%solve(omega.eta)))
+          Uc.y<-compute.LLy(phiMc,Uargs,Dargs,DYF,pres)
+          Uc.eta<-0.5*rowSums(etaMc*(etaMc%*%somega))
           deltu<-Uc.y-U.y+Uc.eta-U.eta
           ind<-which(deltu<(-1)*log(runif(NM)))
           etaM[ind,]<-etaMc[ind,]
@@ -255,13 +260,8 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
           etaMc<-etaM
           etaMc[,vk2]<-etaM[,vk2]+matrix(rnorm(NM*nrs2), ncol=nrs2)%*%mydiag(domega2[vk2,nrs2])
           phiMc[,ind.eta]<-mean.phiM[,ind.eta]+etaMc
-          psiMc<-transphi(phiMc,saemixObject["model"]["transform.par"])
-          fpred<-saemixObject["model"]["model"](psiMc, IdM, XM)
-          for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
-          gpred<-error(fpred,pres,XM$ytype)
-          DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)**2+log(gpred)
-          Uc.y<-colSums(DYF) # Warning: Uc.y, Uc.eta = vecteurs
-          Uc.eta<-0.5*rowSums(etaMc*(etaMc%*%solve(omega.eta)))
+          Uc.y<-compute.LLy(phiMc,Uargs,Dargs,DYF,pres)
+          Uc.eta<-0.5*rowSums(etaMc*(etaMc%*%somega))
           deltu<-Uc.y-U.y+Uc.eta-U.eta
           ind<-which(deltu<(-log(runif(NM))))
           etaM[ind,]<-etaMc[ind,]
@@ -275,18 +275,20 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
     }
     
     phiM[,ind.eta]<-mean.phiM[,ind.eta]+etaM
+    phik<-array(t(phiM),dim=c(nb.parameters,N,nsamp))
     if(k==1) {
-      eik<-array(t(phiM),dim=c(nb.parameters,N,nsamp))
+      eik<-phik
       varik<-0*eik
       ebar[,k,]<-apply(eik,c(1,3),mean)
-      ij<-(k-1)*N
+      ij<-0
       econd[,(ij+1):(ij+N),]<-eik
       sdcond[,(ij+1):(ij+N),]<-varik
     } else {
       eik1<-eik
-      eik<-eik*(k-1)/k+array(t(phiM),dim=c(nb.parameters,N,nsamp))/k
-      varik<-varik*(k-1)/k+(array(t(phiM),dim=c(nb.parameters,N,nsamp))**2)/k+ (eik1**2)*(k-1)/k-(eik**2)
+      eik<-eik*(k-1)/k+phik/k
+      varik<-varik*(k-1)/k+(phik**2)/k + (eik1**2)*(k-1)/k-(eik**2)
       sdik<-sqrt(varik)
+      sdik[sdik<0]<-0
       ij<-(k-1)*N
       econd[,(ij+1):(ij+N),]<-eik
       sdcond[,(ij+1):(ij+N),]<-sdik
@@ -296,48 +298,98 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
         ibeg<-max(1,k-saemixObject["options"]$ipar.lmcmc)
         ekmax<-(ebar[,k,]+abs(ebar[,k,])* saemixObject["options"]$ipar.rmcmc)
         ekmin<-(ebar[,k,]-abs(ebar[,k,])*saemixObject["options"]$ipar.rmcmc)
+        sdkmax<-(sdbar[,k,]+sdbar[,k,]* saemixObject["options"]$ipar.rmcmc)
+        sdkmin<-(sdbar[,k,]-sdbar[,k,]*saemixObject["options"]$ipar.rmcmc)
         if(nsamp==1) {
-          vec<-apply(ebar[,ibeg:k,],1,max)
-          vec2<-apply(ebar[,ibeg:k,],1,min)
-          iek<-sum(vec>ekmax)+sum(vec2<ekmin)
-          vec<-apply(sdbar[,ibeg:k,],1,max)
-          vec2<-apply(sdbar[,ibeg:k,],1,min)
-          sdek<-sum(vec>(sdbar[,k,]+abs(sdbar[,k,])* saemixObject["options"]$ipar.rmcmc))+ sum(vec2<(sdbar[,k,]-abs(sdbar[,k,])* saemixObject["options"]$ipar.rmcmc))
+          maxek<-apply(ebar[,ibeg:k,],1,max)
+          minek<-apply(ebar[,ibeg:k,],1,min)
+          iek<-sum(maxek>ekmax)+sum(minek<ekmin)
+          maxsdk<-apply(sdbar[,ibeg:k,],1,max)
+          minsdk<-apply(sdbar[,ibeg:k,],1,min)
+          sdek<-sum(maxsdk>sdkmax)+ sum(minsdk<sdkmin)
         } else {
-          vec<-apply(ebar[,ibeg:k,],c(1,3),max)
-          vec2<-apply(ebar[,ibeg:k,],c(1,3),min)
-          iek<-sum(vec>ekmax)+sum(vec2<ekmin)
-          if(ibeg==1) ibeg<-2
-          vec<-apply(sdbar[,ibeg:k,],c(1,3),max)
-          vec2<-apply(sdbar[,ibeg:k,],c(1,3),min)
-          sdek<-sum(vec>(sdbar[,k,]+abs(sdbar[,k,])* saemixObject["options"]$ipar.rmcmc))+ sum(vec2<(sdbar[,k,]-abs(sdbar[,k,])* saemixObject["options"]$ipar.rmcmc))
+          maxek<-apply(ebar[,ibeg:k,],c(1,3),max)
+          minek<-apply(ebar[,ibeg:k,],c(1,3),min)
+          iek<-sum(maxek>ekmax)+sum(minek<ekmin)
+          ibeg2<-ifelse(ibeg==1,2,ibeg)
+          maxsdk<-apply(sdbar[,ibeg2:k,],c(1,3),max)
+          minsdk<-apply(sdbar[,ibeg2:k,],c(1,3),min)
+          sdek<-sum(maxsdk>sdkmax)+ sum(minsdk<sdkmin)
         }
-        if(is.na(sdek)) sdek<-0
+        if(is.na(sdek)) sdek<-1
         if(iek==0 & sdek==0) break
       }
     }
-    if((k%%50)==50) {
-      cat(".")
-      if(saemixObject["options"]$displayProgress) {
-        #        par(mfrow=plot.opt$mfrow,ask=plot.opt$ask)
-        par(mfrow=plot.opt$mfrow)
-        limy<-cbind(apply(cbind(apply(ebar[,1:k,],1,min),ekmin),1,min), apply(cbind(apply(ebar[,1:k,],1,max),ekmax),1,max))
-        for(ipar in ind.eta) {
-          laby<-saemixObject["model"]["name.modpar"][ipar]
-          plot(1:k,ebar[ipar,1:k,1],type="n",xlab=plot.opt$xlab, ylab=laby,ylim=limy[ipar,],main=plot.opt$main)
-          for(isamp in 1:nsamp) 
-            lines(1:k,ebar[ipar,1:k,isamp],col=plot.opt$col,lty=plot.opt$lty, lwd=plot.opt$lwd)
-          abline(h=apply(ebar[,k,],1,mean), col=plot.opt$ablinecol, lty=plot.opt$ablinelty,lwd=plot.opt$ablinelwd)
-          abline(h=apply(ekmax,1,mean)[ipar], col=plot.opt$ablinecol, lty=plot.opt$ablinelty,lwd=plot.opt$ablinelwd)
-          abline(h=apply(ekmin,1,mean)[ipar], col=plot.opt$ablinecol, lty=plot.opt$ablinelty,lwd=plot.opt$ablinelwd)
+    if(k>50 & k%%50==1) {
+      if(saemixObject["options"]$warnings) cat(".")
+      #       if(saemixObject["options"]$displayProgress) {
+      #         #        par(mfrow=plot.opt$mfrow,ask=plot.opt$ask)
+      #         par(mfrow=plot.opt$mfrow)
+      #         limy<-cbind(apply(cbind(apply(ebar[,1:k,],1,min),ekmin),1,min), apply(cbind(apply(ebar[,1:k,],1,max),ekmax),1,max))
+      #         for(ipar in ind.eta) {
+      #           laby<-saemixObject["model"]["name.modpar"][ipar]
+      #           plot(1:k,ebar[ipar,1:k,1],type="n",xlab=plot.opt$xlab, ylab=laby,ylim=limy[ipar,],main=plot.opt$main)
+      #           for(isamp in 1:nsamp) 
+      #             lines(1:k,ebar[ipar,1:k,isamp],col=plot.opt$col,lty=plot.opt$lty, lwd=plot.opt$lwd)
+      #           abline(h=apply(ebar[,k,],1,mean), col=plot.opt$ablinecol, lty=plot.opt$ablinelty,lwd=plot.opt$ablinelwd)
+      #           abline(h=apply(ekmax,1,mean)[ipar], col=plot.opt$ablinecol, lty=plot.opt$ablinelty,lwd=plot.opt$ablinelwd)
+      #           abline(h=apply(ekmin,1,mean)[ipar], col=plot.opt$ablinecol, lty=plot.opt$ablinelty,lwd=plot.opt$ablinelwd)
+      #         }
+      #       }
+    }
+  }
+  #  cat("k=",k,"\n")
+  if(saemixObject["options"]$warnings) cat("\n")
+  if(saemixObject["options"]$warnings) {
+    if(k>=kmax){cat("Computing the empirical conditional mean and variance: maximum number of iterations reached without meeting convergence criterion (max.iter=",kmax,")\n") }
+    else cat("Convergence achieved in",k,"iterations\n")
+    
+  }
+  if(saemixObject["options"]$displayProgress) {
+    ibeg<-1
+    nitr<-(k-ibeg)
+    if(FALSE) { # phi-scale
+      ebarbar<-apply(ebar[,ibeg:(k-1),],c(1,2),mean) # mean across the samples => npar x nitr
+      sdbarbar<-apply(sdbar[,ibeg:(k-1),],c(1,2),mean) # mean across the samples => npar x nitr
+      #      sdbarbar<-apply(sdbar,3,mean)
+      ypl1<-data.frame(Iteration=rep((ibeg:(k-1)),nb.parameters),condMean=c(t(ebarbar)),par=rep(saemixObject["model"]["name.modpar"],each=nitr),ymin=rep(ekmin,each=nitr),ymax=rep(ekmax,each=nitr),type="mean")
+      ypl2<-data.frame(Iteration=rep((ibeg:(k-1)),nb.parameters),condMean=c(t(sdbarbar)),par=rep(saemixObject["model"]["name.modpar"],each=nitr),ymin=rep(sdkmin,each=nitr),ymax=rep(sdkmax,each=nitr),type="SD")
+      ypl<-rbind(ypl1,ypl2)
+      # gpl<-ggplot(data=ypl,aes(x=Iteration,y=condMean)) +geom_line() + facet_wrap(as.factor(type)~as.factor(par), scales="free") + geom_ribbon(aes(ymin=ymin,ymax=ymax),alpha=0.2,fill="darkcyan")
+      # print(gpl)
+    } else { # psi-scale
+      nitr<-(k-ibeg+1)
+      for(i in 1:nsamp) {
+        for(itr in ibeg:k) {
+          #          sdbar[,itr,i]<-dtransphi(t(ebar[,itr,i]),saemixObject["model"]["transform.par"])*sdbar[,itr,i]
+          ebar[,itr,i]<-transphi(t(ebar[,itr,i]),saemixObject["model"]["transform.par"])
         }
       }
+      #      tsdkmin<-rowMeans(sdkmin)*dtransphi(t(rowMeans(ekmin)),saemixObject["model"]["transform.par"])
+      #      tsdkmax<-rowMeans(sdkmax)*dtransphi(t(rowMeans(ekmax)),saemixObject["model"]["transform.par"])
+      if(nsamp==1) {
+        tsdkmin<-sdkmin
+        tsdkmax<-sdkmax
+        tekmin<-transphi(t(ekmin),saemixObject["model"]["transform.par"])
+        tekmax<-transphi(t(ekmax),saemixObject["model"]["transform.par"])
+      } else {
+        tsdkmin<-rowMeans(sdkmin)
+        tsdkmax<-rowMeans(sdkmax)
+        tekmin<-transphi(t(rowMeans(ekmin)),saemixObject["model"]["transform.par"])
+        tekmax<-transphi(t(rowMeans(ekmax)),saemixObject["model"]["transform.par"])
+      }
+      
+      ebarbar<-apply(ebar[,ibeg:k,],c(1,2),mean) # mean across the samples => npar x nitr
+      sdbarbar<-apply(sdbar[,ibeg:k,],c(1,2),mean) # mean across the samples => npar x nitr
+      #      sdbarbar<-apply(sdbar,3,mean)
+      ypl1<-data.frame(Iteration=rep(c(ibeg:k),nb.parameters),condMean=c(t(ebarbar)),par=rep(saemixObject["model"]["name.modpar"],each=nitr),ymin=rep(tekmin,each=nitr),ymax=rep(tekmax,each=nitr),type="mean")
+      ypl2<-data.frame(Iteration=rep(c(ibeg:k),nb.parameters),condMean=c(t(sdbarbar)),par=rep(saemixObject["model"]["name.modpar"],each=nitr),ymin=rep(tsdkmin,each=nitr),ymax=rep(tsdkmax,each=nitr),type="SD")
+      ypl<-rbind(ypl1,ypl2)
+      # gpl<-ggplot(data=ypl,aes(x=Iteration,y=condMean)) +geom_line() + facet_wrap(as.factor(type)~as.factor(par), scales="free") + geom_ribbon(aes(ymin=ymin,ymax=ymax),alpha=0.2,fill="darkcyan")
+      # print(gpl)
     }
-    k<-k+1
   }
-  cat("\n")
-  if(k>=kmax)
-    cat("Computing the empirical conditional mean and variance: maximum number of iterations reached without meeting convergence criterion (max.iter=",kmax,")\n") else cat("Convergence achieved in",k,"iterations\n")
+  
   eta.cond<-matrix(0,nrow=dim(etaM)[1],ncol=nb.parameters)
   eta.cond[,ind.eta]<-etaM
   eta.cond<-array(t(eta.cond),dim=c(nb.parameters,N,nsamp))
@@ -345,18 +397,25 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
   cond.shrinkage<-100*(1-apply(eta.cond,2, var)/mydiag(saemixObject["results"]["omega"]))
   names(cond.shrinkage)<-paste("Sh.",names(cond.shrinkage),".%",sep="")
   resh.eik<-resh.varik<-array(0,dim=c(nrow=N,ncol=nb.parameters,nsamp))
-  psi.samp<-resh.eik
+  phi1<-array(t(phiM),dim=c(nb.parameters,N,nsamp))
+  phi.samp<-psi.samp<-array(0,dim=c(N,nb.parameters,nsamp))
   for(isamp in 1:nsamp) {
     resh.eik[,,isamp]<-t(eik[,,isamp])
     resh.varik[,,isamp]<-t(varik[,,isamp])
-    psi.samp[,,isamp]<-transphi(resh.eik[,,isamp],saemixObject["model"]["transform.par"])
+    phi.samp[,,isamp]<-t(phi1[,,isamp])
+    for(isuj in 1:N)
+      psi.samp[isuj,,isamp]<-transphi(t(phi.samp[isuj,,isamp]),saemixObject["model"]["transform.par"])
   }
+  cond.mean.psi<-cond.mean.phi<-apply(resh.eik,c(1,2),mean) # Mean across samples of means of phiM at last iteration
+  for(isuj in 1:N)
+    cond.mean.psi[isuj,]<-transphi(t(cond.mean.psi[isuj,]),saemixObject["model"]["transform.par"])
   
-  saemixObject["results"]["cond.mean.phi"]<-apply(resh.eik,c(1,2),mean)
+  saemixObject["results"]["cond.mean.phi"]<-cond.mean.phi
+  saemixObject["results"]["cond.mean.psi"]<-cond.mean.psi
   saemixObject["results"]["cond.var.phi"]<-apply(resh.varik,c(1,2),mean)
   saemixObject["results"]["cond.mean.eta"]<-eta.cond
   saemixObject["results"]["cond.shrinkage"]<-cond.shrinkage
-  saemixObject["results"]["phi.samp"]<-resh.eik
+  saemixObject["results"]["phi.samp"]<-phi.samp
   saemixObject["results"]["phi.samp.var"]<-resh.varik
   saemixObject["results"]["psi.samp"]<-psi.samp
   
